/*
 * File:   intro.js
 * Author: Thomas Mueller
 * 
 * Numbers are taken from Table A.1, page 545
 * 
 * Planetary fact sheet:
 *   http://nssdc.gsfc.nasa.gov/planetary/factsheet/
 */
"use strict";

function intro() {
    this.e = 1.602176565e-19;  // C
    this.G = 6.67384e-11;      // m^3 kg^-1 s^-2
    this.c = 299792458;        // m / s
    this.Msun = 1.99e30;
    this.AU = 1.495978707e11;  // m
    this.days = 24 * 3600.0;
    
    // Schwarzschild radius
    this.calc_rs = function (mass) {
        return 2.0 * this.G * mass / (this.c * this.c);
    }
    
    // Second cosmic velocity
    this.calc_vk2 = function (mass, radius) {
        var rs = this.calc_rs(mass);
        return this.c * Math.sqrt(rs / radius) * 1e-3;
    }
    
    // Orbital velocity depending on major axis a and current orbital radius r
    this.calc_vis_viva = function (a, r) {
        var mu = this.G * this.Msun;        
        return Math.sqrt(mu * (2./r - 1./a));
    }
    
    // orbital equation r = r(phi)
    this.calc_orbit = function (a, epsilon, phi) {
        return a * (1.0 - epsilon*epsilon) / (1.0 + epsilon * Math.cos(phi));
    }
    
    // Minor axis from major axis and eccentricity
    this.calc_minorAxis = function (a, epsilon) {
        return a * Math.sqrt(1. - epsilon*epsilon);
    }
    
    // Orbital period for Sun and major axis a
    this.calc_period = function (a) {
        var mu = this.G * this.Msun;
        return Math.sqrt(4.0 * Math.PI * Math.PI / mu * Math.pow(a, 3.0));
    }
    
    this.planetaryFactSheet = [
        { objName: "none", majorAxis: 1.0, eccentricity: 0, period: 1.0 },
        { objName: "Merkur",  majorAxis: 0.4, eccentricity: 0.205, period: 88.0 },
        { objName: "Venus", majorAxis: 0.723, eccentricity: 0.007, period: 224.7 },
        { objName: "Erde",  majorAxis: 1.0, eccentricity: 0.017, period: 365.2 },
        { objName: "Mars",  majorAxis: 1.523, eccentricity: 0.094, period: 687.0 },
        { objName: "Jupiter",  majorAxis: 5.2, eccentricity: 0.049, period: 4331.0 },
        { objName: "Saturn",  majorAxis: 9.58, eccentricity: 0.057, period: 10747.0 },
        { objName: "Uranus",  majorAxis: 19.2, eccentricity: 0.046, period: 30589.0 },
        { objName: "Neptun",  majorAxis: 30.05, eccentricity: 0.011, period: 59800.0 },
        { objName: "Pluto",  majorAxis: 39.48, eccentricity: 0.244, period: 90560.0 },
        ];
}
