/*
 * File:   schwarzschild.js
 * Author: Thomas Mueller
 */
"use strict";

function Schwarzschild() {
    this.c = 1.0;
    this.rs = 2.0;
    var eps = 1.0e-8;
    
    var m_lambda = 0.0;
    var m_lambdaStep = 0.01;
    var y = new Array(8);
    
    /* Calculate constant of motion 'k' for lightlike geodesic.
        Args:
          r_obs:   observer position
    */
    this.calc_lr_const_k = function (r_obs) { 
        return Math.sqrt(1.0 - this.rs / r_obs);
    }
    
    
    /* Calculate constant of motion 'h' for lightlike geodesic.
        Args:
          r_obs:  observer position
          ksi:    viewing direction (radians)
     */
    this.calc_lr_const_h = function (r_obs, ksi) {
        return r_obs * Math.sin(ksi);
    }
    
    this.calc_lr_veff = function (r_obs, ksi, r) {
        var h = this.calc_lr_const_h(r_obs, ksi);
        return 0.5 * (1.0 - this.rs / r) * h * h / (r * r);
    }
    
    this.calc_lr_rmin = function (r_obs, ksi) {
        var h = this.calc_lr_const_h(r_obs, ksi);
        var k = this.calc_lr_const_k(r_obs);
        var eps = h / k;
        var sqr3 = Math.sqrt(3.0);        
        return 2.0 / sqr3 * this.c * eps * Math.cos(1.0 / 3.0 * 
            Math.acos(-3.0 * sqr3 * this.rs / (2.0 * this.c * eps)));
    }
    
    this.calc_ksiCrit = function (r_obs) {
        var yobs = this.rs / r_obs;
        var ksi = Math.asin(Math.sqrt(6.75 * yobs * yobs * (1.0 - yobs)));
        if (r_obs < 1.5 * this.rs) {
            ksi = Math.PI - ksi;
        }
        return ksi;
    }
    
    this.calc_lr_vmax = function (r_obs, ksi) {
        return this.calc_lr_veff(r_obs, ksi, 1.5 * this.rs);
    }
    
    this.calc_gamma = function (beta) {
        return 1.0 / Math.sqrt(1.0 - beta * beta);
    }
    
    this.calc_tl_const_k = function (r_obs, beta) {
        var gam = this.calc_gamma(beta);
        return this.c * this.c * gam * Math.sqrt(1.0 - this.rs / r_obs);
    }
    
    this.calc_tl_const_h = function (r_obs, beta, ksi) {
        var gam = this.calc_gamma(beta);
        return this.c * beta * gam * r_obs * Math.sin(ksi);
    }
    
    this.calc_tl_veff = function (r_obs, beta, ksi, r) {
        var h = this.calc_tl_const_h(r_obs, beta, ksi);
        return 0.5 * (1.0 - this.rs / r) * (h * h / (r * r) + this.c * this.c);
    }
    
    /* Position of extrema of the effective potential
       Args:
         r_obs:
         beta:
         ksi:  
         pm:    +1 (minimum),  -1 (maximum)
     */
    this.calc_tl_r_extrema = function (r_obs, beta, ksi, pm) {
        var h = this.calc_tl_const_h(r_obs, beta, ksi);
        var w = h * h - 3.0 * this.c * this.c * this.rs * this.rs;
        return (h * h + pm * h * Math.sqrt(w)) / (this.c * this.c * this.rs);
    }
    
    this.calc_tl_roots = function (r_obs, beta, ksi) {
        var gam = this.calc_gamma(beta);
        var xi = this.rs / r_obs;        
        var alpha = Math.pow(beta * gam * Math.sin(ksi) / xi, 2.0);
        var d = gam * gam * (1.0 - xi) - 1.0;
        
        var p = 1.0/(3.0*alpha) - 1.0/9.0;
        var q = -1.0/27.0 + 1.0/(6.0 * alpha) + 0.5 * d / alpha;
        var r = Math.sign(q) * Math.sqrt(Math.abs(p));
        
        var D = q*q + p*p*p;        
        var x1 = 0.0;
        var x2 = 0.0;
        var x3 = 0.0;
        if (D <= 0) {
            var phi = Math.acos(q / Math.pow(r, 3.0));
            x1 = -2.0 * r * Math.cos(phi/3.0) + 1.0/3.0;
            x2 = 2.0 * r * Math.cos((Math.PI - phi)/3.0) + 1.0/3.0;
            x3 = 2.0 * r * Math.cos((Math.PI + phi)/3.0) + 1.0/3.0;
        }
        else {
            var phi = Math.acosh(q / Math.pow(r, 3.0));
            x1 = -2.0 * r * Math.cosh(phi/3.0) + 1.0/3.0;
            // x2 and x3 are complex
        }
        
        return [this.rs/x1, this.rs/x2, this.rs/x3];
    }
    
    this.getCartCoords = function(y, c) {
        var r = y[1];
        var theta = y[2];
        var phi = y[3];
        c[0] = r * Math.sin(theta) * Math.cos(phi);
        c[1] = r * Math.sin(theta) * Math.sin(phi);
        c[2] = r * Math.cos(theta);
    }

    /**
     * Test break condition for the spacetime
     * @param pos :  current position
     */
    this.breakCondition = function(y) {
        var r = y[1];
        if ((r < 0.0) || (r*r <= (1.0 + eps)*this.rs*this.rs)
            || r > 1000.0) {
            return true;
        }
        return false;
    }
    
    
    this.testConstraint = function(y) {
        var r = y[1];
        var theta = y[2];
        var sth = Math.sin(theta);
        
        var dt = y[4];
        var dr = y[5];
        var dth = y[6];
        var dph = y[7];
        
        var sum = 0.0;
        sum += -(1.0 - this.rs/r)*dt*dt + dr*dr/(1.0-this.rs/r) 
               + r*r*(dth*dth + sth*sth*dph*dph);   
        return (Math.abs(sum) < 1e-4);
    }
    
    this.localToCoord = function(pos, ldir, cdir) {
        var r = pos[1];
        var theta = pos[2];
        var w = Math.sqrt(1.0 - this.rs/r);
        
        cdir[0] = ldir[0] / w;
        cdir[1] = ldir[1] * w;
        cdir[2] = ldir[2] / r;
        cdir[3] = ldir[3] / (r * Math.sin(theta));
        return true;
    }
    
    this.calcDerivs = function(y, dydx) {
        dydx[0] = y[4];
        dydx[1] = y[5];
        dydx[2] = y[6];
        dydx[3] = y[7];
        
        var rs = this.rs;
        var r = y[1];
        var theta = y[2];
        var sth = Math.sin(theta);
        var cth = Math.cos(theta);
        
        dydx[4] = -rs / (r * (r - rs)) * y[4] * y[5];
        dydx[5] = -0.5 * rs * (r - rs) / Math.pow(r, 3.0) * y[4] * y[4] 
            + 0.5 * rs / ( r * (r - rs)) * y[5] * y[5] 
            + (r - rs) * (y[6] * y[6] + sth * sth * y[7] * y[7]);
        dydx[6] = -2.0 / r * y[5] * y[6] + sth * cth * y[7] * y[7];
        dydx[7] = -2.0 / r * y[5] * y[7] - 2.0 * cth / sth * y[6] * y[7];
    }
    
    
    this.calculateGeodesic = function(initPos, initDir, maxNumPoints, points) {
        if (!this.initializeGeodesic(initPos, initDir)) {
            return false;
        }
                
        var breakCond = false;
        var numPoints = 1;
        
        var cc = [];
        this.getCartCoords(y,cc);
        points.push([cc[0],cc[1]]);
        
        while(!breakCond && numPoints < maxNumPoints) {
            if (!this.nextStep()) {
                breakCond = true;
            } else {
                if (this.testConstraint(y)) {
                    var cc = [];
                    this.getCartCoords(y,cc);
                    points.push([cc[0],cc[1]]);
                    numPoints++;
                } else {
                    breakCond = true;
                }
            }
        }
        return true;
    }

    this.nextStep = function() {
        if (this.breakCondition(y)) {
            return false;
        }
        var i;
        var yn = new Array(8);
        var dydx = new Array(8);
        var k1 = new Array(8);
        var k2 = new Array(8);
        var k3 = new Array(8);
        var k4 = new Array(8);
        
        this.calcDerivs(y, dydx);
        for(i=0; i < 8; i++) {
            k1[i] = m_lambdaStep * dydx[i];
            yn[i] = y[i] + 0.5 * k1[i];
        }
        
        this.calcDerivs(yn, dydx);
        for(i = 0; i < 8; i++) {
            k2[i] = m_lambdaStep * dydx[i];
            yn[i] = y[i] + 0.5 * k2[i];
        }

        this.calcDerivs(yn, dydx);
        for(i = 0; i < 8; i++) {
            k3[i] = m_lambdaStep * dydx[i];
            yn[i] = y[i] + k3[i];
        }

        this.calcDerivs(yn, dydx);
        for(i = 0; i < 8; i++) {
            k4[i] = m_lambdaStep * dydx[i];
            yn[i] = y[i] + 1.0 / 6.0 * (k1[i] + 2.0 * k2[i] + 2.0 * k3[i] + k4[i]);
            y[i] = yn[i];
        }
        m_lambda += m_lambdaStep;
        return true;
    }
    
    this.resetAffineParam = function() {
        m_lambda = 0.0;
    }
    
    this.setAffineParamStep = function(lambdaStep) {
        m_lambdaStep = lambdaStep;
    }
    
    /**
     * Initialize geodesic (private method)
     */
    this.initializeGeodesic = function(initPos, initDir) {
        this.resetAffineParam();
        
        this.setInitialPosition(initPos);
        this.setInitialDirection(initDir);
        return this.testConstraint(y);
    }
    
    this.setInitialPosition = function(initPos) {
        for(var i=0; i<4; i++) {
            y[i] = initPos[i];
        }
    }
    
    this.setInitialDirection = function(initDir) {
        var pos = [y[0],y[1],y[2],y[3]];
        var cdir = [0,0,0,0];
        this.localToCoord(pos, initDir, cdir);     
        for(var i=0; i<4; i++) {
            y[4+i] = cdir[i];
        }
    }
}
