/*
 * File:   twinparadox.js
 * Author: Thomas Mueller
 */
"use strict";

function twinparadox() {
    this.acc = 1.0;
    //this.g = 1.0326479;
    this.g = 1.032274079;
    this.c = 1.0;

    /* Calculate coordinate time t depending on the proper time tau
        Args:
          tau1:  proper time for one phase [years]
          tau:   current proper time [years]
        Returns:
          Coordinate time for time span [0,tau] in [years]
    */
    this.t_of_tau = function (tau1, tau) {
        var gdc = this.acc * this.g / this.c;
        var cdg = 1.0 / gdc;
        if (tau >= 0.0 && tau <= tau1) {
            return cdg * Math.sinh(gdc * tau);
        }
        else if (tau > tau1 && tau <= 3.0 * tau1) {
            return cdg * (Math.sinh(gdc * (tau - 2.0 * tau1))
                          + 2.0 * Math.sinh(gdc * tau1));
        }
        else if (tau > 3.0 * tau1 && tau <= 4.0 * tau1) {
            return cdg * (Math.sinh(gdc * (tau - 4.0 * tau1))
                          + 4.0 * Math.sinh(gdc * tau1));
        }
        else {
            return 0.0;
        }
    }

    /* Calculate coordinate position x depending on the proper time tau
        Args:
            tau1:  proper time for one phase [years]
            tau:   current proper time [years]
        Returns:
            Current position [lightyears]
    */
    this.x_of_tau = function (tau1, tau) {
        var gdc = this.acc * this.g / this.c;
        var c2dg = this.c * this.c / (this.acc * this.g);
        if (tau >= 0.0 && tau <= tau1) {
            return c2dg * (Math.cosh(gdc * tau) - 1.0);
        }
        else if (tau > tau1 && tau <= 3.0 * tau1) {
            return c2dg * (2.0 * Math.cosh(gdc * tau1)
                          - Math.cosh(gdc * (tau - 2.0*tau1)) - 1.0);
        }
        else if (tau > 3.0 * tau1 && tau <= 4.0 * tau1) {
            return c2dg * (Math.cosh(gdc * (tau - 4.0*tau1)) - 1.0);
        }
        else {
            return 0.0;
        }
    }

    /* Calculate current velocity beta = v/c
        Args:
          tau1:  proper time for one phase [years]
          tau:   current proper time [years]
        Returns:
          Current velocity beta
    */
    this.beta_of_tau = function (tau1, tau) {
        var gdc = this.acc * this.g / this.c;
        if (tau >= 0.0 && tau <= tau1) {
            return Math.tanh(gdc * tau);
        }
        else if (tau > tau1 && tau <= 3.0 * tau1) {
            return -Math.tanh(gdc * (tau - 2.0*tau1));
        }
        else if (tau > 3.0 * tau1 && tau <= 4 * tau1) {
            return Math.tanh(gdc * (tau - 4.0*tau1))
        }
        else {
            return 0.0;
        }
    }

    this.tau1_of_dist = function (dist) {
        return this.c / (this.acc * this.g)
                * Math.acosh(0.5 * this.acc * this.g * dist / (this.c * this.c) + 1.0);
    }
    
    this.tau1_of_beta = function (beta) {
        return this.c / (this.acc * this.g) * Math.atanh(beta);
    }
    
    this.year_to_date = function (y) {
        var val = y;
        var years = Math.floor(val);
        val = (val - years) * 365.0;
        var days = Math.floor(val);
        val = (val - days) * 24.0;
        var hours = Math.floor(val);
        val = (val - hours) * 60.0;
        var minutes = Math.floor(val);
        val = (val - minutes) * 60.0;
        var seconds = Math.floor(val);
        return [years, days, hours, minutes, seconds];
    }
    
    this.year_to_date_str = function(y) {
        if (isNaN(y)) {
            return "";
        }
        var ds = this.year_to_date(y);
        return ds[0] + "y " + ds[1] + "d " + ds[2] + "h " + ds[3] + "m " + ds[4].toFixed(0) + "s";
    }
}
